@@ -6,7 +6,7 @@ package Business::ES::NIF;
 
 =cut
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 use strict;
 use warnings;
@@ -22,23 +22,23 @@ use warnings;
 
  Dump:
 
-    $VAR1 = bless( {
-                 'status' => 1,
-                 'nif' => '01234567L',
-                 'vies' => 0,
-                 'extra' => 'NIF',
-                 'type' => 'NIF',
-               }, 'NIF' );
+    bless( {
+           'status' => 1,
+           'nif' => '01234567L',
+           'vies' => 0,
+           'extra' => 'NIF',
+           'type' => 'NIF',
+           }, 'NIF' );
 
-    $VAR1 = bless( {
-                 'status' => 0,
-                 'nif' => 'B01234567',
-                 'vies' => 1,
-                 'vies_check' => 0,
-                 'extra' => 'Sociedad Limitada - S.L',
-                 'type' => 'CIF',
-                 'vies_error' => 'Invalid VAT Number (false)'
-               }, 'NIF' );
+    bless( {
+           'status' => 0,
+           'nif' => 'B01234567',
+           'vies' => 1,
+           'vies_check' => 0,
+           'extra' => 'Sociedad Limitada - S.L',
+           'type' => 'CIF',
+           'vies_error' => 'Invalid VAT Number (false)'
+           }, 'NIF' );
 
 =head1 DESCRIPTION
 
@@ -56,7 +56,12 @@ Se puede activar la comprobacion sobre el VIES ( Business::Tax::VAT::Validation
 
 use constant {
 	      NIF_LETRAS => 'TRWAGMYFPDXBNJZSQVHLCKE',
-	      CIF_LETRAS => 'JABCDEFGHI'
+	      CIF_LETRAS => 'JABCDEFGHI',
+	      RE_CIF     => qr/^([ABCDEFGHJPQRUVNW])([0-9]{7})([0-9])$/,
+	      RE_CIFe    => qr/^([SQPK])([0-9]{7})([A-J])$/,
+	      RE_NIF     => qr/^([0-9]{8})([A-Z])$/,
+	      RE_NIFe    => qr/^([KLM])([0-9]{7})([A-Z])$/,
+	      RE_NIE     => qr/^([XYZ])([0-9]{7})([A-Z])$/
 	     };
 
 my %PROVINCIA_CODES = (
@@ -162,87 +167,107 @@ my %PROVINCIA_CODES = (
 
 my $Types = {
 	     CIF => {
-                     re => qr/^[ABCDEFGHJPQRUVNW][0-9]{8}$/,
+                     re  => RE_CIF,
                      val => sub {
-                       shift =~ /^([ABCDEFGHJPQRUVNW])([0-9]{7})([0-9])$/x;
-                       return _validate_cif($1,$2,$3);
+		       $_[0] =~ RE_CIF or return 0;
+                       return _validate_cif($1, $2, $3);
                      },
                      extra => sub {
-                       my $cif = shift;
-                       my $Tipos = {
-                                    'A' => 'Sociedad Anonima - S.A',
-                                    'B' => 'Sociedad Limitada - S.L',
-                                    'C' => 'Sociedad Colectiva - S.C',
-                                    'D' => 'Sociedades comanditarias',
-                                    'E' => 'Comunidad de bienes y herencias',
-                                    'F' => 'Sociedades cooperativas',
-                                    'G' => 'Asociaciones',
-                                    'H' => 'Comunidaddes de propietarios',
-                                    'J' => 'Sociedades civiles',
-                                    'P' => 'Corporaciones locales',
-                                    'Q' => 'Organismos publicos',
-                                    'N' => 'Entidades extranjeras',
-                                    'R' => 'Congregaciones e instituciones religiosas',
-                                    'U' => 'Uniones temporales de epresas',
-                                    'V' => 'Otros tipos de sociedades',
-                                    'W' => 'Establecimientos permanentes de entidades no residentes en España',
-                                   };
-		       
-                       $cif =~ /^([ABCDEFGHJPQRUVNW])[0-9]{7}[0-9]$/x;
-                       
-                       return $Tipos->{$1};
+                       return {
+			       'A' => 'Sociedad Anonima - S.A',
+			       'B' => 'Sociedad Limitada - S.L',
+			       'C' => 'Sociedad Colectiva - S.C',
+			       'D' => 'Sociedades comanditarias',
+			       'E' => 'Comunidad de bienes y herencias',
+			       'F' => 'Sociedades cooperativas',
+			       'G' => 'Asociaciones',
+			       'H' => 'Comunidaddes de propietarios',
+			       'J' => 'Sociedades civiles',
+			       'P' => 'Corporaciones locales',
+			       'Q' => 'Organismos publicos',
+			       'N' => 'Entidades extranjeras',
+			       'R' => 'Congregaciones e instituciones religiosas',
+			       'U' => 'Uniones temporales de epresas',
+			       'V' => 'Otros tipos de sociedades',
+			       'W' => 'Establecimientos permanentes de entidades no residentes en España',
+			      }->{ substr(shift, 0, 1) };
                      }
                     },
 	     CIFe => {
-		      re  => qr/^[SQPK][0-9]{7}[A-J]$/,
+		      re  => RE_CIFe,
 		      val => sub {
-			shift =~ /^([SQPK])([0-9]{7})([A-J])$/x;
+			$_[0] =~ RE_CIFe or return 0;
 			return _validate_cif($1,$2,$3);
 		      },
 		      extra => sub {
-			my $cif = shift;
-			my $Tipos = {
-				     'S' => 'Organos de administracion del estado',
-				     'Q' => 'Organismos autónomos, estatales o no, y asimilados, y congregaciones e instituciones religiosas',
-				     'P' => 'Corporaciones locales.',
-				     'K' => 'Formato antiguo orden EHA/451/2008',
-				    };
-			
-			$cif =~ /^([SQPK])[0-9]{7}[A-J]$/x;
-			
-			return $Tipos->{$1};
+			return {
+				'S' => 'Órganos de la Administración del Estado',
+				'Q' => 'Organismos autónomos o instituciones religiosas',
+				'P' => 'Corporaciones locales',
+				'K' => 'Formato antiguo (DNI sin letra de control)',
+			       }->{ substr(shift, 0, 1) };
 		      }
 		     },
 	     NIF => {
-                     re    => qr/^[0-9]{8}[A-Z]$/,
-                     val   => sub { return _validate_nif(shift); },
-                     extra => sub { return 'NIF'; }
+                     re    => RE_NIF,
+		     extra => sub { return 'NIF'; },
+                     val   => sub { return _validate_nif(shift); }
                     },
+	     NIFe => {
+		      re    => RE_NIFe,
+		      extra => sub { return 'NIF Especial (KLM)'; },
+		      val   => sub {
+			shift =~ /^[KLM]([0-9]{7})([A-Z])$/ or return 0;
+			return substr(NIF_LETRAS, $1 % 23, 1) eq $2 ? 1 : 0;
+		      }
+		     },
 	     NIE => {
-		     re    => qr/^[XYZ][0-9]{7}[A-Z]$/,
-		     val   => sub { return _validate_nie(shift); },
-		     extra => sub { return 'NIE'; }
+		     re    => RE_NIE,
+		     extra => sub { return 'NIE'; },
+		     val   => sub { return _validate_nie(shift); }
 		    }
-};
+	    };
+
+=head2 type
+
+Devuelve el tipo (NIF, NIE, CIF, CIFe, NIFe).
+
+    my $tipo = $obj->type;
+
+=head2 extra
+
+Devuelve una descripcion extra del tipo, por ejemplo:
+
+"Sociedad Anonima - S.A", "NIE", "NIF Especial (KLM)", etc.
+
+    my $desc = $obj->extra;
+
+=head2 error
+
+Devuelve el mensaje de error (si existe) cuando el NIF no es valido.
+
+    warn $obj->error unless $obj->is_valid;
+
+=cut
+
+sub type  { return shift->{type};  }
+sub extra { return shift->{extra}; }
+sub error { return shift->{error}; }
 
 sub _cif_provincia {
   my $cif = shift;
-    
-  # Validar entrada
+
   return unless defined $cif;
-    
-  # Extraer código de provincia (posiciones 1-2, base 0)
+
   my $provincia = substr($cif, 1, 2);
-    
-  # Validar que son dígitos
+
   return unless $provincia =~ /^[0-9]{2}$/;
-    
-  # Buscar provincia
+
   return $PROVINCIA_CODES{$provincia};
 }
 
 sub _validate_cif {
-  my ($sociedad, $inscripcion, $control) = (shift,shift,shift);
+  my ($sociedad, $inscripcion, $control) = @_;
 
   my @n = split //, $inscripcion;
   my $pares = $n[1] + $n[3] + $n[5];
@@ -268,17 +293,13 @@ sub _validate_cif {
 }
 
 sub _validate_nif {
-  shift =~ /^([0-9]{8})([A-Z])$/x;
-
-  my ($NIF,$DC) = ($1,$2);
-  my $L = substr( NIF_LETRAS , $NIF % 23, 1);
-
-  return $L eq $DC ? 1 : 0;
+  shift =~ RE_NIF;
+  return substr( NIF_LETRAS , $1 % 23, 1) eq $2 ? 1 : 0;
 }
 
 sub _validate_nie {
-  shift =~ /^([XYZ])([0-9]{7})([A-Z])$/x;
-                       
+  shift =~ RE_NIE;
+
   my ($NIE,$NIF,$DC) = ($1,$2,$3);
                        
   for ($NIE) {
@@ -287,9 +308,7 @@ sub _validate_nie {
     $NIF = '2'.$NIF if /Z/;
   }
   
-  my $L = substr( NIF_LETRAS , $NIF % 23, 1);
-
-  return $L eq $DC ? 1 : 0;
+  return substr( NIF_LETRAS , $NIF % 23, 1) eq $DC ? 1 : 0;
 }
 
 =head2 new
@@ -302,9 +321,9 @@ sub new {
   my ($class, %args) = @_;
 
   my $self = {
-	      nif => '',
+	      nif    => '',
 	      status => 0,
-	      vies => $args{vies} // 0,
+	      vies   => $args{vies} // 0
 	     };
   
   $self = bless $self, $class;
@@ -317,7 +336,8 @@ sub new {
 =head2 set
 
     Set NIF
-    $vies = 1 || 0
+
+    $vies = 1 | 0
 
 =cut
 
@@ -326,17 +346,16 @@ sub set {
   my $nif  = shift;
   my $vies = shift // 0;
 
-  $self->{vies} = $vies if $vies;
+  $self->{vies} = 1 if $vies;
 
   unless (defined $nif) {
     $self->{status} = 0;
-    $self->{error} = "NIF vacío";
+    $self->{error}  = "NIF vacío";
     return;
   }
 
-  $nif =~ s/[-\.\s]//g;
-  $self->{nif} = uc $nif;
-  
+  $self->{nif} = uc($nif =~ tr/-. //dr);
+
   delete $self->{nif_check} if $self->{nif_check};
 
   $self->check();
@@ -379,6 +398,8 @@ sub check {
 
 =head2 vies
 
+Valida el NIF contra el sistema europeo VIES. Devuelve 1 si es correct, 0 si no lo es.
+
 =cut
 
 sub vies {
